#include "stock.h"

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>
#include <pwd.h>


static pthread_t async_th;
int cash=100000;   /* This should be provided through a server request */

cid_t max_companies;
int16_t max_simult_quot;
int16_t max_history;

int server, server2;
req_t ok_s='c', err='e', ok_r;
int passwd;
char *name;
plid_t plid = -1;

static char *sold = "Sold";
static char *bought = "Bought";
static char *last_trade = "Last Trade";
static char *dividend = "Dividend";

void login_newplayer ();
void re_entry();
void sync_channel ();		       /* stuff through port 5000 */
void *async_channel ();		       /* stuff through 5001 */

/* Trap Ctrl-C */

void cleanup() {
   puts ("Caught Ctrl-C");
   quit ();  
   _exit(0);
}

/* ignore SIGPIPE so that program does not die on write trouble */

void ignore_sigpipe () {
   struct sigaction sig;

   sig.sa_handler = SIG_IGN;
   sig.sa_flags = 0;
   sigemptyset(&sig.sa_mask);
   sigaction(SIGPIPE,&sig,NULL);
}

/* connect to server */

int make_connection (char *address, unsigned short portnum) {
   struct sockaddr_in sa;
   int s;
   
   sa.sin_family = AF_INET;
   sa.sin_port = htons(portnum);
   sa.sin_addr = *(struct in_addr *) address;
   
   if ((s = socket (AF_INET, SOCK_STREAM, 0)) < 0)
     return (-1);
   
   if (connect(s, &sa, sizeof(sa)) < 0)
     return (-1);

   return s;
}

/* call the shots */

int main (int ac, char *av[]) {
   
   signal (SIGINT, cleanup);
   ignore_sigpipe();
      
   sync_channel (ac, av);
   return 0;
}

void display_results (char *s1, cid_t cid, char *s3, char *s4) {
   char *s[4];

   s[0] = s1;
   if (cid >=0) s[1] = company[cid].name;
   else s[1] = NULL;
   s[2] = s3;
   s[3] = s4;

   update_cash ();
   update_result_area (s);
}

/* Handle all incoming replies to requests on the second channel */

void *async_channel () {
   req_t req;
   qty_t qty;
   cid_t cid;
   pr_t pr;

   while (1) {
      char *s1, *s2;
      sr_req2 (req);
      
      s1 = (char *) malloc (8 * sizeof (char));
      s2 = (char *) malloc (8 * sizeof (char));
      
      switch (req) {
       case POST_BUY_QUOT:
	 sr_cid2 (cid);
	 sr_qty2 (qty);
	 sr_pr2 (pr);
	 RECV_OK2;
	 cash -= qty * pr;
	 update_new_buy (cid, qty);
	 sprintf (s1, "%d", qty);
	 sprintf (s2, "%d", pr);
	 display_results (bought, cid, s1, s2);
	 break;
	 
       case POST_SELL_QUOT:
	 sr_cid2 (cid);
	 sr_qty2 (qty);
	 sr_pr2 (pr);
	 RECV_OK2;
	 cash += qty * pr;	 
	 update_new_sell (cid, qty);
	 sprintf (s1, "%d", qty);
	 sprintf (s2, "%d", pr);
	 display_results (sold, cid, s1, s2);
	 break;
	 
       case DIVIDEND:
	 sr_cid2 (cid);
	 sr_qty2 (qty);
	 sr_pr2 (pr);
	 RECV_OK2;
	 cash += pr;
	 sprintf (s2, "%d", pr);
	 display_results (dividend, -1, NULL, s2);
	 break;
	 
       default:
	 printf ("screwup\n");
	 break;
      
      } /* switch */
   }  /* while (1) */
} /* channel2 */


void sync_channel (int ac, char *av[]) {
   req_t req = NEW_LOGIN2;
   struct hostent *hostinfo;
        
   if (ac < 2) {
      printf ("Usage : %s hostname\n", av[0]);
      exit(0);
   }
   
   if ((hostinfo = gethostbyname (av[1])) == NULL) {
      printf ("Unknown host : %s\n", av[1]);
      exit (1);
   }
   
   if ((server = make_connection(hostinfo->h_addr_list[0], PORTNUM)) < 0) {
      perror ("make_connection");
      exit (1);
   }
   
   login_newplayer();
   
   if ((server2 = make_connection(hostinfo->h_addr_list[0], PORTNUM+1)) < 0) {
      perror ("make_connection");
      exit (1);
   }
   
   sw_char (server2, req);
   sw_int (server2, passwd);
   RECV_OK2;
   
   if (ok_r == 'c') {
      printf ("Connected and Logged in\n");

      if (pthread_create (&async_th, NULL, async_channel, NULL) != 0)
	printf ("Trouble with threads\n");
      ui_gtk_entry (ac, av);
   }
   else {
      printf ("Trouble\n");
      exit (2);
   }
   printf ("Trouble logging in.. bad password ?\n");
   exit (1);
}

/* post sell or buy requests */
   
void quote (a, b, c, d) {
   req_t req_buy = POST_BUY_QUOT, req_sell = POST_SELL_QUOT;
   char ch;
   
   if (d == POST_BUY_QUOT) sw_req (req_buy);
   else sw_req (req_sell);

   sw_cid (a);
   sw_pr (b);
   sw_qty (c);
   
   sr_char (server, ch);
   if (ch == 'e') printf ("screwed\n");
   RECV_OK;
   return;
}

/* New Player Login */

void login_newplayer() {
   req_t req=NEW_LOGIN;
   char *t;
   short int len;

   name = (char *) malloc (32);
   printf ("Enter login name : ");
   scanf ("%31s", name);
   len = (short) strlen (name) + 1;
   
   t = (char *) malloc (16);
   t = getpass ("Enter password (numbers only) : ");
   passwd = atoi (t);
   free (t);
   
   sw_req (req);
   sw_int (server, passwd);
   
   sw_s_int (server, len);
   sw (server, name, len);
   
   sr_int (server, plid);
   
#ifdef DEBUG
   printf ("plid %d\n", plid);
#endif
   
   RECV_OK ;
}

/* Connect again, if already logged in once. */

void re_entry() {
   req_t req=RE_ENTRY;
   
   sw_req (req);
   sw_int (server, passwd);
   sw_int (server, plid);
   RECV_OK ;
}

inline static void send_req_for_info (req_t req2) {
   req_t req1=COMP_INFO;
   
   sw_req (req1);
   sw_req (req2);
}

void get_comp_misc () {
   register int i;
   
   send_req_for_info (COMP_MISC);
   
   sr_cid (max_companies);
   sr_s_int (server, max_simult_quot);
   sr_s_int (server, max_history);
   RECV_OK;
   
   company = (company_t *) malloc ((max_companies+1) * sizeof (company_t));
   
   for (i=0; i<=max_companies; ++i)
     company[i].history = NULL;
   
}

void get_comp_names () {
   int i=0;
   char *name;

   send_req_for_info (COMP_NAME);
   
   sw_cid (i);
   sw_cid (max_companies);
   
   RECV_OK;
   for (i=0; i<=max_companies; ++i) {
      name = sock_read_string (server);      
      company[i].name = name;
      update_buy_list (name);

#ifdef DEBUG
      printf ("%s\n", name);
#endif      
   }
   RECV_OK;
}
   
void get_comp_last_trade (cid_t cid) {
   char *s1;
   
   send_req_for_info (COMP_LAST_TRADE_1);
   
   sw_cid (cid);
   RECV_OK;
   sr_pr (company[cid].last_trade);
   RECV_OK;
   
   s1 = (char *) malloc (8);
   sprintf (s1, "%d", company[cid].last_trade);
   display_results (last_trade, cid, NULL, s1);
   free (s1);
}

void get_comp_hist (cid_t cid) {
   register int i;
   company_t *c = &company[cid];
   
   send_req_for_info (COMP_HIST_1);
   
   sw_cid (cid);
   RECV_OK;
   
   if (c->history == NULL) 
     c->history = (pr_t *) malloc (sizeof(pr_t) * max_history);
   
   for (i=0; i<max_history; ++i)
     sr_pr (c->history[i]);

   RECV_OK;
}
      
void get_comp_list (cid_t cid, int call) {
   register int j;
   char *s1, *s2;
      
   if (call == BUY) {
      send_req_for_info (COMP_BUY_LIST_1);            
      display_results ("Buy List", cid, NULL, NULL);
   }
   else {
      send_req_for_info (COMP_SELL_LIST_1);
      display_results ("Sell List", cid, NULL, NULL);
   }
   
   sw_cid (cid);
   RECV_OK;

   s1 = (char *) malloc (8);
   s2 = (char *) malloc (8);
   
   for (j=0; j<max_companies; ++j) {
      pr_t pr;
      qty_t qty;
      
      sr_pr (pr);
      sr_qty (qty);
      
      sprintf (s1, "%d", qty);
      sprintf (s2, "%d", pr);
      
      display_results (NULL, -1, s1, s2);
      free (s1);
      free (s2);
      
   }
   RECV_OK;
}

