#include "stock.h"
#include "threads.h"

#define QUEUE(fn)  if (addto_req_list (client, plid, cid, fn)) \
                       {SEND_OK;} else {SEND_ERR;}

int server2;

void *serve_requests_sync (void *arg) {
   int client;
   req_t req;
   cid_t cid;
   plid_t plid;   
   thread_arg_t *thread_arg;
   player_t *p;
   
   thread_arg = (thread_arg_t *) arg;
   client = thread_arg->client;
      
   plid = do_login (client, thread_arg->th);
   free (thread_arg); 
   do_login2 (plid);
   
   p = &player[plid];
   
   while (1) {
      sr_req (req);
      p->timeout = -timeout; 
      
#ifdef LOG
      fprintf (logfile, "player : %d\t req : %d\n", plid, req);
#endif

      switch (req) {
	 
       case LOGOUT:                   /* disconnect from game, reentry later */
	 kick_player ();
	 break;
	 
       case COMP_INFO:
	 send_company_info (client);
	 break;
	 
       case POST_BUY_QUOT:
	 sr_cid (cid);
	 QUEUE (buy_pr_quote);
	 SEND_OK;
	 break;
	 
       case CHANGE_BUY_QUOT:
	 sr_cid (cid);	 
	 QUEUE (buy_pr_quote_change);
	 SEND_OK;
	 break;
	 
       case POST_SELL_QUOT:
	 sr_cid (cid);
	 QUEUE (sell_pr_quote);
	 SEND_OK;
	 break;
	 
       case CHANGE_SELL_QUOT:
	 sr_cid (cid);	 
	 QUEUE (sell_pr_quote_change);
	 SEND_OK;
	 break;
	 
       default:
#ifdef DEBUG
	 printf ("plid %d : Bullshit request\n", plid);
#endif
	 kick_player ();
	 break;
      }  /* switch (req) */
   }    /* while (1) */
}      /* serve_request () */


void *serve_requests_async (void *arg) {
   thread_arg_t *thread_arg;
   int client;
   int id = 'a'; /* async channel identifier */
   pthread_mutex_t *result_mut;
   pthread_cond_t *result_cond;
   player_t *p;
   
   thread_arg = (thread_arg_t *) arg;
   client = thread_arg->client;
   p = &player[thread_arg->plid];
   
   p->async_th = thread_arg->th;
   
   pthread_setspecific (plid_TSD, (void *) thread_arg->plid);
   free (thread_arg);
   pthread_setspecific (id_TSD, (void *) id);
   
   result_mut = &p->result_mut;
   result_cond = &p->result_cond;

   
   while (1) {
      result_list_t *head;
      result_list_t *tail;
      
      pthread_cleanup_push (pthread_mutex_unlock, (void *) result_mut);
      pthread_mutex_lock (result_mut);
      pthread_cond_wait (result_cond, (void *) result_mut);
      pthread_cleanup_pop (1);
      
      p->timeout = -timeout;
      
      head = p->head;
      tail = p->tail;
      
      while (head != tail) {

	 sw_req (head->req);
	 sw_cid (head->cid);
	 sw_qty (head->qty);
	 sw_pr (head->pr);
	 
	 head = head->next;
	 SEND_OK;
      }
      p->head = head;
   } /* while (1) */
} /* serve_request_async () */
